#!/usr/bin/perl -w
# Ensembl の pep にある ENSP レコードに対応する ENST レコードをとってくる
# アミノ酸配列のインファイルと各種 ENSP ID に対応するデータベースをハッシュとして書き込む
# cDNA db は巨大なので，時間がかかります．3 レコードのファイルで 20 秒位でしょうか．

# perl ensDNAseqRet.pl AAseq.fas

use strict;
use warnings;

my $AAseqFile = "$ARGV[0]";
my $outNucFile = "outNuc_".$AAseqFile;

# ENSP とデータベースの対応をハッシュに書き込む．
my %dbNameHashP = ( 
                    'ENSP' => 'db_fol/HumanCDNAdb.fas',
                    'ENSCINP' => 'db_fol/cionCDNAdb.fas',
                    'ENSDARP' => 'db_fol/DanioCDNAdb.fas',
                    'ENSTNIP' => 'db_fol/TetraodonCDNAdb.fas',
#                    'DAMMY' => 'db_fol/DammyCDNAdb.fas',

#                    'ENSCINP' => '/Volumes/HD-PEBU2/999Sato09DB_fol/RawDataEns61legacy/Ciona_intestinalis.JGI2.61.cdna.all.fa',
#                    'ENSDARP' => '/Volumes/HD-PEBU2/999Sato09DB_fol/RawDataEns61legacy/Danio_rerio.Zv9.61.cdna.all.fa',
#                    'ENSORLP' => '/Volumes/HD-PEBU2/999Sato09DB_fol/RawDataEns61legacy/Drosophila_melanogaster.BDGP5.25.61.cdna.all.fa',
#                    'ENSGALP' => '/Volumes/HD-PEBU2/999Sato09DB_fol/RawDataEns61legacy/Gallus_gallus.WASHUC2.61.cdna.all.fa',
#                    'ENSGACP' => '/Volumes/HD-PEBU2/999Sato09DB_fol/RawDataEns61legacy/Gasterosteus_aculeatus.BROADS1.61.cdna.all.fa',
#                    'ENSP'    => '/Volumes/HD-PEBU2/999Sato09DB_fol/RawDataEns61legacy/Homo_sapiens.GRCh37.61.cdna.all.fa',
#                    'ENSORLP' => '/Volumes/HD-PEBU2/999Sato09DB_fol/RawDataEns61legacy/Oryzias_latipes.MEDAKA1.61.cdna.all.fa',
#                    'ENSTNIP' => '/Volumes/HD-PEBU2/999Sato09DB_fol/RawDataEns61legacy/Tetraodon_nigroviridis.TETRAODON8.61.cdna.all.fa',
#                    'ENSXETP' => '/Volumes/HD-PEBU2/999Sato09DB_fol/RawDataEns61legacy/Xenopus_tropicalis.JGI4.1.61.cdna.all.fa'
);

# データベースのファイルがあるか調べる．ない場合は，解析を終了する．
foreach (keys %dbNameHashP) {
  if(-e "$dbNameHashP{$_}"){
#    print "$dbNameHashP{$_} was FOUND\n";
    next;
  } else {
    print "Data base, $dbNameHashP{$_}, was NOT FOUND\n";
    print "program stopped\n";
    exit;
  }
}


# %dbNameHashP　の Name である ENSP を ENST に書き換える
my %dbNameHashT = ( );  
foreach (keys %dbNameHashP) {
  my $tmp = $_;
  $tmp =~ s/P$//g;
  $tmp = $tmp."T";
  $dbNameHashT{$tmp} = $dbNameHashP{$_};
}

# アミノ酸配列のファイルを読み込む
open(AASEQ,"$AAseqFile") or die"$!";
my @aaSeqFile = <AASEQ>;
close AASEQ;

# アミノ酸配列のファイルをハッシュと配列に格納する．
my ($ref_fasHashAA,$ref_nameArrayAA, $seqLengthAA) = &fasHashReader(\@aaSeqFile);

# インファイルの形式が Ensembl のままであるか調べる．違えば終了
foreach (@$ref_nameArrayAA) {
  if($_ =~ /transcript:EN/){
    next;
  } else {
    print "The name line of $AAseqFile is not Ensembl format\n";
    print "  $_\n";
    print "The name line should be written as\n";
    print "  >ENSP00000374981 pep:known chromosome:GRCh37:14:106053226:106054732:-1 gene:ENSG00000211890 transcript:ENST00000390539\n";
    print "program stopped.\n";
    exit;
  }
}


# enst ID と用いるデータベースをサブルーチンcDNAseqRetにわたして，選ばれた cDNA レコードをプリントアウト
open(OUT,">$outNucFile");
foreach (@$ref_nameArrayAA){
  $_ =~ /:(\w+)$/;                           # アミノ酸ファイルの name line から enst ID をとってくる．
  my $enst = $1;
  my $enstNoNum = $enst;
  $enstNoNum =~ s/\d//g;
  foreach (keys %dbNameHashT) {              # $enst に対応するデータベース名を %dbNameHashT から選ぶ
    if ($enstNoNum eq $_) {                     # $enst = ENST00000390539，%dbNameHashT = ('ENST' => 'db_fol/HumanCDNAdb.fas')
      my ($ref_nucNamesSelected,$ref_nucRecSelected) = &cDNAseqRet($enst,$dbNameHashT{$_});
      my @selectedNucName = @$ref_nucNamesSelected;
      my %selectedNucRec = %$ref_nucRecSelected;
      print " ".@selectedNucName." $enst was found in $dbNameHashT{$_}\n";
      foreach (@selectedNucName) {
         print OUT "$_\n$selectedNucRec{$_}\n";
       }
    }
  }
}
print "\n";
close OUT;


exit;


#### サブルーチン ####
### DB から cDNA 配列のレコードをとってくる．
sub cDNAseqRet {

  my ($enst,$dbFile) = @_;
  my @nucNamesSelected = ( );   # 選んだ cDNA の名前
  my %nucRecSelected = ( );    # 選んだ cDNA レコードのハッシュ
  
  
  # データベースファイルを開ける．
  open(INFILE,"$dbFile") or die "$!";
  my @inDBCont = <INFILE>;
  close INFILE;
  
  my ($ref_fasHash,$ref_nameArray, $seqLength) = &fasHashReader(\@inDBCont);
  
  @nucNamesSelected = grep(/>$enst/,@$ref_nameArray);    #cDNA データベースに enst ID を含むレコードがいくつかるか調べる
  foreach (@nucNamesSelected) {
    $nucRecSelected{$_} = $$ref_fasHash{$_} ;
  }
  return(\@nucNamesSelected,\%nucRecSelected);    
}

#  Fasta 形式をハッシュに読み込む Subroutine
sub fasHashReader
{
  my $name = "";
  my $seq = "";
  my $seqLength = "";
  my $stone = 0;
  my @nameArraySR = ( );
  my %fasHashSR = ( );

  my ($dbFileContSR) = @_;

  foreach my $dbFileLine (@$dbFileContSR) {
    chomp($dbFileLine);
    if($dbFileLine =~ /^>/){  
      if($stone == 0){$name = $dbFileLine;
        $stone = 1;  
      } else {
        $fasHashSR{$name} = $seq;  
        push(@nameArraySR,$name);
        $name = "$dbFileLine";
        $seq = "";
      }
    } else {
      $seq .= $dbFileLine; 
    }
  }
  $fasHashSR{$name} = $seq;
  $seqLength = length($seq);
  push(@nameArraySR,$name);

  # ハッシュと配列をメインプログラムに返す．
  return (\%fasHashSR,\@nameArraySR,\$seqLength);

}